@@ -19,7 +19,7 @@
 */
 
 #ifndef AKONADI_CALFILTERPROXYMODEL_P_H
-#define AKOANDI_CALFILTERPROXYMODEL_P_H
+#define AKONADI_CALFILTERPROXYMODEL_P_H
 
 #include <QSortFilterProxyModel>
 
