@@ -29,6 +29,9 @@
 #include <kcalcore/journal.h>
 #include <kcalcore/todo.h>
 #include <klocale.h>
+#include <klocalizedstring.h>
+#include <kglobal.h>
+#include <kicon.h>
 
 #include <QItemSelectionModel>
 
