@@ -44,7 +44,7 @@ struct ProtocolHelperValuePool
   MimeTypePool mimeTypePool;
   QHash<Collection::Id, Collection> ancestorCollections;
 };
-  
+
 /**
   @internal
   Helper methods for converting between libakonadi objects and their protocol
@@ -135,7 +135,7 @@ class ProtocolHelper
           rv += command;
           rv += ' ';
         }
-        QList<typename T::Id>  uids;
+        QVector<typename T::Id>  uids;
         foreach ( const T &object, objects )
           uids << object.id();
         ImapSet set;
