@@ -129,8 +129,9 @@ class KCALCORE_EXPORT Event : public Incidence
     /**
       Sets whether the event has an end date/time.
       @param b If set, indicates the event has an end date.
+      @deprecated Use setDtEnd( KDateTime() )
     */
-    void setHasEndDate( bool b );
+    KDE_DEPRECATED void setHasEndDate( bool b );
 
     /**
       Returns whether the event has an end date/time.
@@ -199,7 +200,7 @@ class KCALCORE_EXPORT Event : public Incidence
 
     /**
        @copydoc
-       IncidenceBase::iconName()
+       Incidence::iconName()
     */
     QLatin1String iconName( const KDateTime &recurrenceId = KDateTime() ) const;
 
@@ -250,7 +251,9 @@ class KCALCORE_EXPORT Event : public Incidence
 
 } // namespace KCalCore
 
+//@cond PRIVATE
 Q_DECLARE_TYPEINFO( KCalCore::Event::Ptr, Q_MOVABLE_TYPE );
+//@endcond
 
 //@cond PRIVATE
 namespace KPIMUtils {
