@@ -19,7 +19,7 @@
 
 #include "myrightsjob.h"
 
-#include <KDE/KLocale>
+#include <KDE/KLocalizedString>
 #include <KDE/KDebug>
 
 #include "acljobbase_p.h"
@@ -80,4 +80,3 @@ Acl::Rights MyRightsJob::rights()
   Q_D( MyRightsJob );
   return d->myRights;
 }
-
